/****************************************************************************/
/*  File Name:          AT17.C							    */                                       
/*  Title:              UART and Two-wire implementation file               */
/*  Last Updated:       August 5, 2001						    */
/*  Target Device:      AT94K05AL/10AL/40AL-25DQC (208-pin packet only)     */ 
/*												    */
/*   Support Email:      fpslic@atmel.com						    */
/*   Support Hotline:    +1 (408) 436-4119 (USA)				    */
/****************************************************************************/

#include "at17.h"

/****************************************************************************/
/*  initAT94K         									    */
/****************************************************************************/

void initAT94K(void)
{
      
        DDRD = 0xFF;                           /* Configure PORTD as Output */
        PORTD = 0x00;                          /* Dispaly 0x00 on PORTD     */



        PORTE |= (1 << PE7);                   /* PE7 High, SER_EN Disabled */
        DDRE  |= (1 << PE7);                   /* PE7 Output                */

        EIMF  = 0x33;                          /* Enalbe External Interrrupt*/
        SREG |= 0x80;		                 /* Enable Interrupts         */
       

}



/****************************************************************************/
/* set interrupt handler for External Hardware interrupt zero               */
/****************************************************************************/

#pragma interrupt_handler   INT0_handler:EXT_INT0

/****************************************************************************/
/* Interrupt Service Routine                                                */
/* This routine is used to execute the reconfiguraion procedure             */
/****************************************************************************/

void INT0_handler(void)
{

	unsigned char status;
	unsigned long resetAddress = raddress;
	unsigned char byteCount = 0;
	unsigned char readBuffer[T_BYTE];
	unsigned char rxvalue_buffer[TR_BYTE];
      unsigned char program_data[TR_BYTE];
 	unsigned char value2 = bad;


/****************************************************************************/
/* Initialize receiving buffer             	                            */
/****************************************************************************/


      for(byteCount = 0; byteCount < TR_BYTE; byteCount++)
      { 
          rxvalue_buffer[byteCount] = 0;
          program_data[byteCount] = 0;
      }


/****************************************************************************/
/* Initialize I/O ports, UART0 and Two-wire Interface port                  */
/****************************************************************************/

                     
      initAT94K();                    /* Setup PORTD and PORTE              */
	

      initUART0(BAUD19K2);            /* Baud Rate set to 9600 bps          */      
       
                                      /* using a 4 MHz Crystal */
      initTWS(0);                     /* Initialize TWS of AT84K @ 250 kHz  */
    
        

      PORTD = 0x01;                   /* Turn on LED for PORT D to 
                                         indicate the chip is ready         */    
     
//      PORTE &= ~(1 << PE7);   	 /* SER_EN = LOW  
//                                     (EEPROM Programming Mode Enabled)    */



/****************************************************************************/
/* Send C character to sender to start data transmission.                   */
/****************************************************************************/
        
      waitMs(255);                    /* Wait for 20ms  delay                */

						 /* Wait for data Regiester to be empty */
      while(!(UCSR0A & (1 << UDRE0))); 

      UDR0 = C_character;            /* Receiver ready, send "C" to sender  */

      


/****************************************************************************/
/* Receive 133 bytes of data. 1st packet contains file info                 */
/****************************************************************************/
	     	
	       		       /* Wait for 133 bytes of data to be received */


      for (byteCount = 0; byteCount < TR_BYTE; byteCount++)
      { 
            
             while (!(UCSR0A & (1<<RXC0)));                  
                
             rxvalue_buffer[byteCount] = UDR0;                 
		   
      }
	  
	


      PORTD = 0x02;                          /* Light up the LED on PORTD   */


      status = validate_packet(&rxvalue_buffer[0]); /* check packet status  */
       

      respond(status);                        /* Send ACK or NAK to sender  */

    

   
/****************************************************************************/
/* if status is not good, (NAK already sent) wait for packet 1 again        */
/****************************************************************************/


      while (status != good)                  /* Check packet status        */
      {
                               /* Wait for 133 bytes of data to be received */
		
            for (byteCount = 0; byteCount < TR_BYTE; byteCount++)
            { 
                
              while (!(UCSR0A & (1<<RXC0)));        
                
              rxvalue_buffer[byteCount] = UDR0;     
                  
             
		   
            }
							    /* Validate 2nd packet again  */

      status = validate_packet(&rxvalue_buffer[0]); 
 
    if (status == good )         
	
           respond(status);                   /* Send ACK or NAK to sender  */
      }
    

        PORTD = 0x05;   			    /* Light up PORTD             */




/****************************************************************************/
/* Send a 2nd "C" character to start actual data transmission               */
/****************************************************************************/
	   
					        /* Wait for data register to be empty */						 

      while(!(UCSR0A & (1 << UDRE0)));  

      UDR0 = C_character;             /* Receiver ready, send "C" to sender */
    

                               /* Wait for 133 bytes of data to be received */

      for (byteCount = 0; byteCount < TR_BYTE; byteCount++)
      { 
             
           while (!(UCSR0A & (1<<RXC0)));              
                
           rxvalue_buffer[byteCount] = UDR0;           
           
           program_data[byteCount] = bit_swap(&rxvalue_buffer[byteCount]); 
		   
      }


      status = validate_packet(&rxvalue_buffer[0]);  /* Valid packet        */
    


/****************************************************************************/
/* program 128 bytes of data to the AT17LV010 EEPROM if status is good      */
/****************************************************************************/


      if (status == good )

      {
       PORTE &= ~(1 << PE7);   	 /* SER_EN = LOW  
                                      (EEPROM Programming Mode Enabled)    */
		
          pageAddress = 0;
          
           
            
          writePage(&program_data[3], 
                   ((unsigned long) (pageAddress << 7) & 0x0001FF80));  

      }		

      

                                 
      respond(status);                         /* Send ACK or NAK to sender */

      
                            
/****************************************************************************/
/* if status != good,  wait for packet again (NAK is already sent).         */
/****************************************************************************/    

     
      while (status != good) 

      {
       
                               /* Wait for 133 bytes of data to be received */
 
       for (byteCount = 0; byteCount < TR_BYTE; byteCount++)
            
       { 
            while (!(UCSR0A & (1<<RXC0)));
                
            rxvalue_buffer[byteCount] = UDR0;
           
            program_data[byteCount] = bit_swap(&rxvalue_buffer[byteCount]); 		        

       }

                               
      status = validate_packet(&rxvalue_buffer[0]);   /* Validate packet    */
        
      
/****************************************************************************/
/* program 128 bytes of data to the AT17LV010 EEPROM if status is good      */
/****************************************************************************/


      if (status == good )         
	{
	   PORTE &= ~(1 << PE7);   	 /* SER_EN = LOW  
                                      (EEPROM Programming Mode Enabled)    */

            pageAddress = 0;
        
            writePage(&program_data[3], 
                     ((unsigned long) (pageAddress << 7) & 0x0001FF80));  
      }


      respond(status);                   /*   Send ACK or NAK to sender     */
    
     
      }                                  /* Loop back if status is no good  */
    
    
   
/****************************************************************************/
/* get remainder of the file                                                */
/****************************************************************************/

                               /* Wait for 133 bytes of data to be received */

      while (status != end) {
     
      for (byteCount = 0; byteCount < TR_BYTE; byteCount++)
      { 
    
           while (!(UCSR0A & (1<<RXC0)));
                
           rxvalue_buffer[byteCount] = UDR0;
		        
         program_data[byteCount] = bit_swap(&rxvalue_buffer[byteCount]); 
	     
         if (rxvalue_buffer[0] == EOT )
		        
               byteCount = TR_BYTE;
		
   
		   
      }

        
        
      status = validate_packet(&rxvalue_buffer[0]);  /* validate the packet */
        
    
   
/****************************************************************************/
/* program 128 bytes of data to the AT17LV010 EEPROM if status is good      */
/****************************************************************************/     
                

      if (status == good )
	{
    PORTE &= ~(1 << PE7);   	 /* SER_EN = LOW  
                                      (EEPROM Programming Mode Enabled)    */
	    
          pageAddress ++;
       
          writePage(&program_data[3], 
                   ((unsigned long) (pageAddress << 7) & 0x0001FF80));  
        
        }
	    
       
      respond(status);                                  /* Send ACK or NAK  */
        
      } 


/****************************************************************************/
/* end of file transmission      		         	                      */
/****************************************************************************/
           
       
      PORTD = 0xAA;



/****************************************************************************/
/* program reset polarity level to low (*RESET/OE)                          */
/* This procedure can be igored if the reset poarity of EEPROM              */
/* is previously programmed.                                                */
/*************************** ************************************************/
          
      PORTE &= ~(1 << PE7);   	 /* SER_EN = LOW  
                                      (EEPROM Programming Mode Enabled)    */

   
   //   programResetPolarity(LOW, resetAddress);

   //   if (verifyResetPolarity(resetAddress))
   //   {
                value2 = good;
   //   }     

    

/****************************************************************************/
/* Check final result from LED on PORTD                                     */
/****************************************************************************/

     /* if programming reset procedure is delted,  "&& (value2 = good)"     */
     /* should be also deleted from the following statement.                */
     


    /*  if (( status == end) && (value2 == good) ) */
      
	   if (status == end)   
          PORTD = 0xFF;
    
      else
     
          PORTD = 0x0F;


/****************************************************************************/	 
/* Turn on configuration mode to start configuration (SER_EN = High)        */
/****************************************************************************/  
	 
        
      PORTE |= 0x80;


        
  }



/****************************************************************************/
/* initTWS         									    */
/****************************************************************************/

void initTWS(unsigned char bitRate)
{
        TWBR  = bitRate;                        /* Setting TWC Bit Rate    */
        TWS_MASTER_INIT();                      /* Included in TWS.H       */


/***************************************************************************/
/* Two-wire Serial Interface Bit Rate Table for 4 MHz                      */
/*								                           */
/*      +-------+-------------+----------+----------+				   */
/*      |       |    TWBR     |      Bit Rate       |                      */
/*      |       +------+------+----------+----------+                      */
/*      | Clock | Hex  | Dec  |    Hz    |    kHz   |                      */
/*      +-------+------+------+----------+----------+                      */
/*      | 4 MHz | 0x00 |   0  | 250000   | 250      |                      */
/*      |       | 0x02 |   2  | 200000   | 200      |                      */
/*      |       | 0x08 |   8  | 125000   | 125      |                      */
/*      |       | 0x0C |  12  | 100000   | 100      |                      */
/*      |       | 0x11 |  17  |  80000   |  80      |                      */
/*      |       | 0x18 |  24  |  62500   |  62.5    |                      */
/*      |       | 0x20 |  32  |  50000   |  50      |                      */
/*      |       | 0x2A |  42  |  40000   |  40      |                      */
/*      |       | 0x38 |  56  |  31250   |  31.25   |                      */
/*      |       | 0x48 |  72  |  25000   |  25      |                      */
/*      |       | 0x5C |  92  |  20000   |  20      |                      */
/*      |       | 0x78 | 120  |  15625   |  15.625  |                      */
/*      |       | 0x98 | 152  |  12500   |  12.5    |                      */
/*      |       | 0xC0 | 192  |  10000   |  10      |                      */
/*      |       | 0xF8 | 248  |   7812.5 |   7.8125 |                      */
/*      +-------+------+------+----------+----------+                      */
/***************************************************************************/
}


/***************************************************************************/
/* initUART0               				                           */
/***************************************************************************/

void initUART0(unsigned int baudRate)
{

        UCSR0B |= ((1 << RXEN0) | (1 << TXEN0) );  /* Enable UART1 RX & TX */
        UCSR0B &= ~(1 << CHR90);               /* Disable 9-Bit Characters */ 

        UBRRHI |= ((baudRate >> 8) & 0x000F);      /* Set BaudRate[11:8]   */
        UBRR0  |= ((baudRate) & 0x00FF);           /* Set BaudRate[7:0]    */
}





/***************************************************************************/
/*  Bit_swap fucntion                                                      */
/***************************************************************************/

unsigned char bit_swap(unsigned char *ptr)
{

   unsigned char swap_data;   
   unsigned char bit_0_reg;
   unsigned char bit_1_reg;
   unsigned char bit_2_reg;
   unsigned char bit_3_reg;
   unsigned char bit_4_reg;
   unsigned char bit_5_reg;
   unsigned char bit_6_reg;
   unsigned char bit_7_reg;
 
  
   bit_0_reg = *ptr;
   bit_1_reg = *ptr;
   bit_2_reg = *ptr;
   bit_3_reg = *ptr;
   bit_4_reg = *ptr;
   bit_5_reg = *ptr;
   bit_6_reg = *ptr; 
   bit_7_reg = *ptr;

   swap_data = (((bit_0_reg & 0x80 ) >> 7) | 
                ((bit_1_reg & 0x40 ) >> 5) |
                ((bit_2_reg & 0x20 ) >> 3) |
                ((bit_3_reg & 0x10 ) >> 1) |
                ((bit_4_reg & 0x08 ) << 1) |
                ((bit_5_reg & 0x04 ) << 3) |
                ((bit_6_reg & 0x02 ) << 5) |
                ((bit_7_reg & 0x01 ) << 7) );


 
   return (swap_data);

}






/***************************************************************************/
/* Delay                                                                   */
/***************************************************************************/

void waitMs(unsigned char ms)
{
 
unsigned char count;

  for(count = 0; count < ms; count ++)
  {

   TCNT0 = 0xC0;
   TCCR0 = 0x03;
   while (!(TIFR & ( 1 << TOV0)));
   TCCR0 = 0x00;
   TIFR |= (1 << TOV0);
  }
}



/***************************************************************************/
/* Calcrc                                                                  */
/***************************************************************************/

int calcrc(unsigned char *ptr, int count)
{

   int crc, i;

   crc = 0;

   while (--count >= 0) {

      crc = crc ^ (int) *ptr++ << 8;
      for (i = 0; i < 8; ++i)
         if (crc & 0x8000)
            crc = crc << 1 ^ 0x1021;
         else
            crc = crc << 1;
   }

return (crc);

}


/***************************************************************************/
/* validate_packet                                                         */
/***************************************************************************/

unsigned char validate_packet(unsigned char *bufptr) 
{

unsigned char packet;
int crc;
                     
packet = bad;

            
  if (bufptr[0] == SOH)          /* Check if it is the start of the packet */

                                 /* Check sequential packet numer          */

  {
   if (bufptr[1] == ((packet_number + 1) & 0xff))  
   {                                             
 
      if ((bufptr[1] + bufptr[2]) == 0xff)  
      {   
               
      crc = calcrc(&bufptr[3],128);          /* Compute CRC and valide it  */
      
         if ((bufptr[131] == (unsigned char)(crc >> 8)) && 
            (bufptr[132] == (unsigned char)(crc))) 

         {    

             packet_number++; 
                            
             packet = good;
 
         }    
                    
       } 
                
    }   
            
   


/* if the packet is the previous packet, donot increament packet number    */
                  
   else if (bufptr[1] == ((packet_number) & 0xff))   
       
            packet = dup; 
  }   
            
            
/***************************************************************************/
/* validate_packet                                                         */
/***************************************************************************/
                      

   else if (bufptr[0] == EOT)          /* Check if it is end of the packet */
                
            packet = end;

   else
            packet = bad;


    return (packet);
        
}




/***************************************************************************/
/* respond  	                                                         */
/***************************************************************************/ 

void respond(unsigned char packet) 
{
    
    if ((packet == good) || (packet == end)) 
    {
          while(!(UCSR0A & (1 << UDRE0)));

              UDR0 = ACK;
			  
           
    }
    else 
    {

        while(!(UCSR0A & (1 << UDRE0)));

              
              UDR0 = NAK;
    }
}



/***************************************************************************/
/* ProgramResetPolarity                                                    */
/*							                                 */
/* Address location 0x20000H through 0x20003H are to program               */
/* reset polarity level of the AT17C/LV010 EEPROM.   	         	         */
/*												   */
/* 0x00 => Active HIGH Reset and Active LOW Output Enable			   */
/* 0xFF => Active LOW Reset and Active HIGH Output Enable			   */
/*												   */
/* So, the memory location values determine the Reset Polarity.  After	   */
/* programming the 20 ms timeout method is used to determine the end of    */
/* the internal programming cycle.                                         */
/***************************************************************************/

void programResetPolarity(unsigned char newResetPolarityLevel, 
unsigned long resetAddress)
{
        unsigned char byteCount;
        unsigned char addressCount;

        /* Send START Condition */
        START_CONDITION();
        WAIT();

        if (TWSR == START)
        {
                /* Send Device Address */
                SEND_BYTE(AT17LV010 & ~(WRITE));
                CLEAR_TWINT();
                WAIT();

                /* Send Memory Address */
                for(addressCount = 0; addressCount < A_BYTE; addressCount++)
                {
                        if( (TWSR == MT_SLA_ACK) || (TWSR == MT_DATA_ACK) )
    
                    {
    SEND_BYTE((unsigned char)(resetAddress >> (((A_BYTE - 1) - addressCount) 
                 * 8 )& 0x000000FF));

                        CLEAR_TWINT();
                        WAIT();
                        }
                }

                /* Program Reset Polarity */
                for(byteCount = 0; byteCount < R_BYTE; byteCount++)
                {
                        if(TWSR == MT_DATA_ACK)
                        {
                                if(newResetPolarityLevel == LOW)
                                        SEND_BYTE(0xFF);
                                else SEND_BYTE(0x00);

                                CLEAR_TWINT();
                                WAIT();
                        }
                }

                /* Send STOP Condition */
                if(TWSR == MT_DATA_ACK)
                {
                        STOP_CONDITION();

                        /* Configurator Timeout -> 20 ms
                         * 4 MHz / 1024 = 3906.25 Hz
                         * 3906.25 Hz = 256 us
                         * 20 ms / 256 us = 78.125
                         * 65536 - 78.125 = 65457.875 = 0xFFB1
                         * Interrupt on 0xFFFF to 0x0000 Transition */

                        TCNT1H = 0xFF;
                        TCNT1L = 0xB1;
                        TCCR1B = 0x05;
                   
                        while(!(TIFR & (1 << TOV1)));
						TCCR1B = 0x00;
						TIFR |= (1 << TOV1);
                }
        }
}



/***************************************************************************/
/* readPage                      							   */
/*												   */
/* Read 128 bytes at address into bufptr.  Starts reading at address 0     */
/* within the page.  Please refer to the Application Note entitled,        */
/* "Programming Specification for Atmel's AT17 and AT17A Series FPGA       */
/* Configuration EEPROMs" found at http://www.atmel.com for detailed       */
/* device address decoding and page address formatting.                    */
/***************************************************************************/

void readPage(unsigned char * bufptr, unsigned long pageAddress)
{
        unsigned char byteCount;
        unsigned char addressCount;

        /* Send START Condition */
        START_CONDITION();
        WAIT();

        if (TWSR == START)
        {
                /* Send Device Address */
                SEND_BYTE(AT17LV010 & ~(WRITE));
                CLEAR_TWINT();
                WAIT(); 

                /* Send Memory Address */
                for(addressCount = 0; addressCount < A_BYTE; addressCount++)
                {
                        if( (TWSR == MT_SLA_ACK) || (TWSR == MT_DATA_ACK) )
                        {
   SEND_BYTE((unsigned char)(pageAddress >> ((( A_BYTE - 1) - addressCount) 
              * 8 ) & 0x000000FF));
                      
					   CLEAR_TWINT();
                       WAIT();   
                        }
                }

                if(TWSR == MT_DATA_ACK)
                {
                        /* Send START Condition */
                        REP_START_CONDITION();
                        WAIT();  

                        if (TWSR == REP_START)
                        {
                                /* Send Device Address */
                                SEND_BYTE(AT17LV010 | READ);
                                CLEAR_TWINT();
                                WAIT();  

                                if(TWSR == MR_SLA_ACK)
                                {
                                        SEND_ACK();
                                        WAIT();   

                                        /* Receive Data */
                                for(byteCount = 0; byteCount < (T_BYTE - 2); 
								byteCount++)
                                        {
                                                if(TWSR == MR_DATA_ACK)
                                                {
                                                   GET_BYTE(bufptr[byteCount]);
                                                        SEND_ACK();
                                                        WAIT();   
                                                }
                                        }

                          /* Receive Last 2 Bytes and Send STOP Condition */
                                        if(TWSR == MR_DATA_ACK)
                                        {
                                                GET_BYTE(bufptr[T_BYTE - 2]);
                                                CLEAR_TWINT();
                                                WAIT();   
                                                if(TWSR == MR_DATA_NACK)
                                                {
                                                  GET_BYTE(bufptr[T_BYTE - 1]);

                                                        STOP_CONDITION();
                                                }
                                        }
                                }
                        }
                }
        }
}


/***************************************************************************/
/* writePage										   */
/*												   */
/* Writes 128 bytes at address from bufptr.  Starts writing at address 0   */
/* within the page.  Please refer to the Application Note entitled,        */
/* "Programming Specification for Atmel's AT17 and AT17A Series FPGA       */
/* Configuration EEPROMs" found at http://www.atmel.com for detailed       */
/* device address decoding and page address formatting.                    */
/* 												   */
/* After programming the 20 ms timeout method is used to determine         */
/* the end of the internal programming cycle.    				   */
/***************************************************************************/

void writePage(unsigned char * bufptr, unsigned long pageAddress)
{
        unsigned char byteCount;
        unsigned char addressCount;

        /* Send START Condition */
        START_CONDITION();
        WAIT();   
        if (TWSR == START)
        {
                /* Send Device Address */
                SEND_BYTE(AT17LV010 & ~(WRITE));
                CLEAR_TWINT();
                WAIT();   
                /* Send Memory Address */
                for(addressCount = 0; addressCount < A_BYTE; addressCount++)
                {
                        if( (TWSR == MT_SLA_ACK) || (TWSR == MT_DATA_ACK) )
                        {
SEND_BYTE((unsigned char)(pageAddress >> (((A_BYTE - 1) - addressCount) * 8)
           & 0x000000FF));
                        CLEAR_TWINT();
                        WAIT();                           }
                }

                /* Send Data */
                for(byteCount = 0; byteCount < T_BYTE; byteCount++)
                {
                        if(TWSR == MT_DATA_ACK)
                        {
                                SEND_BYTE(bufptr[byteCount]);
                                CLEAR_TWINT();
                                WAIT();                           }
                }

                /* Send STOP Condition */
                if(TWSR == MT_DATA_ACK)
                {
                        STOP_CONDITION();


                        /* Configurator Timeout -> 20 ms
                         * 4 MHz / 1024 = 3906.25 Hz
                         * 3906.25 Hz = 256 us
                         * 20 ms / 256 us = 78.125
                         * 65536 - 78.125 = 65457.875 = 0xFFB1
                         * Interrupt on 0xFFFF to 0x0000 Transition */

                        TCNT1H = 0xFF;
                        TCNT1L = 0xB1;
                        TCCR1B = 0x05;
						
					 /* The following delay is needed when */
					/* you continuesly program each page  */
					/* without 20ms delay                 */	
						
                    //    while(!(TIFR & (1 << TOV1)));
					//	TCCR1B = 0x00;
					//	TIFR |= (1 << TOV1);
                }
        }
}


/***************************************************************************/
/* verifyResetPolarity									   */
/*												   */
/* 4 bytes are read from locations 0x20000 through 0x20003.  The 		   */
/* bytes are verified to be of all the same value.  If the are then        */
/* the value is returned.								   */
/*												   */
/* Returned Value 0x00 => Active HIGH Reset and Active LOW Output Enable   */
/* Returned Value 0xFF => Active LOW Reset and Active HIGH Output Enable   */
/*												   */
/* If they are not the same then 0xAA is returned to signal an		   */
/* ERROR condition.									   */
/***************************************************************************/

unsigned char verifyResetPolarity(unsigned long resetAddress)
{
        unsigned char byteCount;
        unsigned char addressCount;
        unsigned char resetPolarity[R_BYTE];
        unsigned char polarity = 0xAA;

        /* Send START Condition */
        START_CONDITION();
        WAIT();   

        if (TWSR == START)
        {
                /* Send Device Address */
                SEND_BYTE(AT17LV010 & ~(WRITE));
                CLEAR_TWINT();
                WAIT();  
                /* Send Memory Address */
                for(addressCount = 0; addressCount < A_BYTE; addressCount++)
                {
                        if( (TWSR == MT_SLA_ACK) || (TWSR == MT_DATA_ACK) )
                        {
  SEND_BYTE( (unsigned char)(resetAddress >> ( ((A_BYTE - 1) - addressCount) 
              * 8 ) & 0x000000FF));
                    CLEAR_TWINT();
                    WAIT();                           }
                }

                if(TWSR == MT_DATA_ACK)
                {
                        /* Send Repeat START Condition */
                        REP_START_CONDITION();
                        WAIT();   
                        if (TWSR == REP_START)
                        {
                                /* Send Device Address */
                                SEND_BYTE(AT17LV010 | READ);
                                CLEAR_TWINT();
                                WAIT();   

                                if(TWSR == MR_SLA_ACK)
                                {
                                        SEND_ACK();
                                        WAIT();  
                                        /* Receive Data */
                    for(byteCount = 0; byteCount < (R_BYTE - 2); byteCount++)
                                        {
                                                if(TWSR == MR_DATA_ACK)
                                                {
                                            GET_BYTE(resetPolarity[byteCount]);
                                                        SEND_ACK();
                                                        WAIT();             }
                                        }
										
										
	               /* Receive Last 2 Bytes and Send STOP Condition */
                                        if(TWSR == MR_DATA_ACK)
                                        {
                                          GET_BYTE(resetPolarity[R_BYTE - 2]);
                                                CLEAR_TWINT();
                                                WAIT();   
                                                if(TWSR == MR_DATA_NACK)
                                                {
                                          GET_BYTE(resetPolarity[R_BYTE - 1]);
                                                        STOP_CONDITION();
                                                }
                                        }
                                }
                        }
                }
        }

        /* Compare Received Reset Polarity */
        for(byteCount = 0; byteCount < (R_BYTE - 1); byteCount++)
        {
                if(resetPolarity[byteCount] == resetPolarity[byteCount + 1])
                        polarity = (resetPolarity[byteCount]);
                else polarity = 0xAA;
        }

        return (polarity);
}


